
#include <string.h>
#include "common.h"

#ifdef _arch_dreamcast
	#include <dc/fmath.h>
	/*#define sin(a) fsin(a)
	#define cos(a) fcos(a)*/
	#define sqrt(a) fsqrt(a)
	
#define fisincos(a,sine,cosine) \
({	asm(	"lds	%2,fpul\n\t" \
		".short	0xf0fd\n\t" \
		"fmov	fr0,%0\n\t" \
		"fmov	fr1,%1\n\t" \
	: "=f" (sine), "=f" (cosine) \
	: "r" (a) \
	: "fpul", "fr0", "fr1"); \
	 })
	
#endif

Vector2D Vector2D::rotate(float angledegree) const
{
    float rad = (float)(-angledegree) * (float)0.017453292519943295555555555555556;
    float crad = cos(rad), srad = sin(rad);
    float fx = (float)x, fy = (float)y;
    return Vector2D((int)(fx*crad - fy*srad),(int)(fy*crad+fx*srad));
}

FVector2D FVector2D::rotate(float angledegree) const
{
    float rad = (float)(-angledegree) * (float)0.017453292519943295555555555555556;
    float crad = cosf(rad), srad = sinf(rad);
    float fx = (float)x, fy = (float)y;
    return FVector2D(fx*crad - fy*srad,fy*crad+fx*srad);
}

FVector2D FVector2D::rotate(Angle angle) const
{
#ifdef _arch_dreamcast
	//float rad = (float)(-angledegree) * (float)0.017453292519943295555555555555556;
	float crad, srad;// = cosf(rad), srad = sinf(rad);
	fisincos(angle,srad,crad);
	float fx = (float)x, fy = (float)y;
	return FVector2D(fx*crad-fy*srad, fy*crad+fx*srad);
#else
	assert(0);
	float rad = (float)(-angle)  / 10430.37835f;
	float crad = cosf(rad), srad = sinf(rad);
	float fx = (float)x, fy = (float)y;
	return FVector2D(fx*crad - fy*srad,fy*crad+fx*srad);
#endif
}

IVector2D DVector2D::toIVec() const
{
	IVector2D tmp =  IVector2D(x >> DVector2DShift, y >> DVector2DShift);
	return tmp;
}

DVector2D IVector2D::toDVec() const
{
	DVector2D tmp =  DVector2D(x << DVector2DShift, y << DVector2DShift);
	return tmp;
}

FVector2D DVector2D::toFVec() const
{
	FVector2D tmp =  FVector2D(DScalarToFScalar(x),DScalarToFScalar(y));
	return tmp;
}

FVector2D DVector2D::toTruncFVec() const
{
	FVector2D tmp =  FVector2D(x >> DVector2DShift, y >> DVector2DShift);
	return tmp;
}

FVector2D IVector2D::toFVec() const
{
	FVector2D tmp =  FVector2D(x, y);
	return tmp;
}

Rect Rect::normalize()
{
	Vector2D newpt1, newpt2;
	newpt1.x = min(pt1.x, pt2.x);
	newpt1.y = min(pt1.y, pt2.y);
	newpt2.x = max(pt1.x, pt2.x);
	newpt2.y = max(pt1.y, pt2.y);	
	pt1 = newpt1;
	pt2 = newpt2;
	return *this;
}

Rect Rect::intersection(Rect r) const
{	//TODO test
	Rect inter;
	inter.pt1.x = max(r.pt1.x, pt1.x);
	inter.pt1.y = max(r.pt1.y, pt1.y);
	inter.pt2.x = min(r.pt2.x, pt2.x);
	inter.pt2.y = min(r.pt2.y, pt2.y);
	return inter;
}


