#ifndef _COMMON_ASM_GUARD
#define _COMMON_ASM_GUARD

#ifdef __ASSEMBLER__
//ASSEMBLER ONLY------------------------------------------

#define ALIGN4BYTE .align 2
#define ALIGN32BYTE .align 5

#define PVR_TA_INPUT		0x10000000	/* TA command input */
#define PVR_RAM_BASE		0xa5000000	/* PVR RAM (raw) */
#define PVR_RAM_INT_BASE	0xa4000000	/* PVR RAM (interleaved) */
#define PVR_RAM_SIZE		(8*1024*1024)	/* RAM size in bytes */
#define PVR_RAM_TOP		(PVR_RAM_BASE + PVR_RAM_SIZE)		/* Top of raw PVR RAM */
#define PVR_RAM_INT_TOP		(PVR_RAM_INT_BASE + PVR_RAM_SIZE)	/* Top of int PVR RAM */

!!!!!! load float from struct, destroys r0
.macro _fmovr0_ __off,__src,__dst
	mov	#\__off,r0
	fmov	@(r0,\__src),\__dst
.endm

!!!!!! three operand fmac
.macro _fmac3_ __sa,__sb,__dst
	.if \__sa == fr0
		.print "ERROR: shouldn't be using _fmac3_ here! use normal fmac."
		.err
	.endif
	fmov	\__sa,fr0
	fmac	fr0,\__sb,\__dst
.endm

!!!!!! multiply decrement
.macro _fmdc_ __sa,__sb,__dst
	.if \__sa <> fr0
		.print "ERROR: _fmdc_ without first fr0 parameter"
		.err
	.endif
	.if (\__dst == fr0) || (\__dst == \__sb)
		.print "ERROR: _fmdc_ dst can't be same as a source reg"
		.err
	.endif
	fneg	\__dst
	fmac	fr0,\__sb,\__dst
.endm

!!!!!! skip parameter instruction on t = 1
.macro _skipt_ __args:vararg
	bt 57182$f
	\__args
57182$ :
.endm

!!!!!! skip parameter instruction on t = 0
.macro _skipf_ __args:vararg
	bf 57183$f
	\__args
57183$ :
.endm

!!!!!! save xmtrx to memory with predecrement
.macro _save8_xmtrx_ __dst
	.irp	dreg, 14,12,10,8, 6,4,2,0
		fmov xd\dreg, @-\__dst
	.endr
.endm

!!!!!! load xmtrx from memory with postincrement
.macro _load8_xmtrx_ __src
	.irp	dreg,0,2,4,6,8,10,12,14
		fmov @\__src+, xd\dreg
	.endr
.endm

!!!!!! put a stack pointer aligned to an 8-byte boundary and toggle FS bit
.macro _8aligned_stack_r0_fschg_
	!get 8-aligned stack
	mov r15,r0
	or #7,r0
	
	fschg
	add #-7,r0
.endm

!!!!! moves the front registers to the back, fschg must be set!
.macro _move_front_to_xmtrx_
	fmov dr0,xd0
	fmov dr2,xd2
	fmov dr4,xd4
	fmov dr6,xd6
	fmov dr8,xd8
	fmov dr10,xd10
	fmov dr12,xd12
	fmov dr14,xd14
.endm

.macro _dump_regs_
	trapa	#1
.endm

.macro _dump_floats_
	trapa	#2
.endm

.macro _dump_all_floats_
	trapa	#3
.endm

#define fm00 fr0
#define fm01 fr1
#define fm02 fr2
#define fm03 fr3
#define fm10 fr4
#define fm11 fr5
#define fm12 fr6
#define fm13 fr7
#define fm20 fr8
#define fm21 fr9
#define fm22 fr10
#define fm23 fr11
#define fm30 fr12
#define fm31 fr13
#define fm32 fr14
#define fm33 fr15

#define dbm00 xd0
#define dbm02 xd2
#define dbm10 xd4
#define dbm12 xd6
#define dbm20 xd8
#define dbm22 xd10
#define dbm30 xd12
#define dbm32 xd14

#define dm00 dr0
#define dm02 dr2
#define dm10 dr4
#define dm12 dr6
#define dm20 dr8
#define dm22 dr10
#define dm30 dr12
#define dm32 dr14

#endif	/* #ifdef __ASSEMBLER__ */

#endif



