#ifndef _COMMON_CPP_GUARD
#define _COMMON_CPP_GUARD


//FUNCTIONS WRITTEN IN C PLUS PLUS----------------------------

#include <math.h>
#include <LinearMath/btVector3.h>

class IVector2D;
class DVector2D;
class FVector2D;

class Vector2D
{
public:
	int x, y;
	inline Vector2D() : x(0) , y(0) { ; }
	inline Vector2D(int startx, int starty) : x(startx) , y(starty) { ; }
	inline int area() const { return x*y; }
	inline int length() const { return (int)sqrt( (float)((float)x*(float)x+(float)y*(float)y) ); }
	inline int lengthsqr() const { long long int tx = x, ty = y; return (tx*tx+ty*ty); }
	Vector2D rotate(float angledegree) const;
	Vector2D rotate(Angle angle) const;

};

inline Vector2D operator+  (const Vector2D& left, const Vector2D& right)
{ return Vector2D(left.x + right.x, left.y + right.y); }
inline Vector2D operator- (const Vector2D& right)
{ return Vector2D(0 - right.x, 0 - right.y); }
inline Vector2D operator- (const Vector2D& left, const Vector2D& right)
{ return Vector2D(left.x - right.x, left.y - right.y); }
inline Vector2D operator* (const Vector2D& left, const Vector2D& right)
{ return Vector2D(left.x * right.x, left.y * right.y); }
inline Vector2D operator* (const Vector2D& left, const int right)
{ return Vector2D(left.x * right, left.y * right); }
inline Vector2D operator* (const Vector2D& left, const float right)
{ return Vector2D((int)(left.x * right), (int)(left.y * right)); }
inline Vector2D operator/ (const Vector2D& left, const int right)
{ return Vector2D(left.x / right, left.y / right); }
inline Vector2D operator/ (const Vector2D& left, const Vector2D& right)
{ return Vector2D(left.x / right.x, left.y / right.y); }
inline Vector2D operator>> (const Vector2D& left, const int right)
{ return Vector2D(left.x >> right, left.y >> right); }
inline Vector2D operator<< (const Vector2D& left, const int right)
{ return Vector2D(left.x << right, left.y << right); }
inline Vector2D operator& (const Vector2D& left, const int right)
{ return Vector2D(left.x & right, left.y & right); }

class DVector2D : public Vector2D
{
public:
	DVector2D() { ; } ;
	inline DVector2D(int startx, int starty) : Vector2D(startx,starty) { ; };
	inline DVector2D(float startx, float starty) : 
		Vector2D(	(int)((float)startx * (float)(1<<DVector2DShift)),
					(int)((float)starty * (float)(1<<DVector2DShift))	)
					{ ; } ;
	//inline DVector2D(int startx, int starty) { x = startx; y = starty; };
	//inline DVector2D(float startx, float starty) {
	//	x = (int)((float)startx * (float)(1<<DVector2DShift)); 
	//	y = (int)((float)starty * (float)(1<<DVector2DShift)); 
	//};
	inline DVector2D(Vector2D src) { x = src.x; y = src.y; };
	IVector2D toIVec() const;
	FVector2D toFVec() const;
	FVector2D toTruncFVec() const;
};

inline float DScalarToFScalar(int val)
{
	return val / (float)(1 << DVector2DShift);
}

class IVector2D : public Vector2D
{
public:
	IVector2D() { ; } ;
	inline IVector2D(int startx, int starty) : Vector2D(startx,starty) { ; };
	inline IVector2D(Vector2D src) : Vector2D(src) { ; };
	//inline IVector2D(int startx, int starty) { x = startx; y = starty; };
	//inline IVector2D(Vector2D src) { x = src.x; y = src.y; };
	DVector2D toDVec() const;
	FVector2D toFVec() const;
	operator DVector2D() { return toDVec(); }
};


class Rect
{
public:
	//in normalized rect, pt1 is lower left, pt2 is upper right
	Vector2D pt1, pt2;

	Rect() : pt1(0,0), pt2(0,0) { ; }
	Rect(Vector2D a, Vector2D b) : pt1(a), pt2(b) { ; }
	Rect(int a[4] ) { pt1 = Vector2D(a[0],a[1]); pt2 = Vector2D(a[2],a[3]); }
	Rect(int a, int b, int c, int d ) { pt1 = Vector2D(a,b); pt2 = Vector2D(c,d); }
	Rect scale(float s) const { return Rect( pt1 * s, pt2 * s ); }
	Rect scale(int s) const { return Rect( pt1 * s, pt2 * s ); }
	Rect shift(int s) const { return Rect( pt1 << s, pt2 << s ); }
	Rect translate(const Vector2D & offset) const { return Rect( pt1 + offset, pt2 + offset ); }
	Rect normalize();
	Rect intersection(Rect r) const;
	int width() const { return pt2.x - pt1.x; };
	int height() const { return pt2.y - pt1.y; };
	int top() const { return pt2.y; }
	int bottom() const { return pt1.y; }
	int right() const { return pt2.x; }
	int left() const { return pt1.x; }
	
	Vector2D topright() const { return pt2; };
	Vector2D bottomleft() const { return pt1; };
	Vector2D topleft() const { return Vector2D(pt1.x,pt2.y); };
	Vector2D bottomright() const { return Vector2D(pt2.x,pt1.y); };
};

typedef Rect IRect;
typedef Rect DRect;

inline Rect operator>> (const Rect& left, const int right)
{ return Rect(left.pt1 << right, left.pt2 << right ); }

typedef unsigned int F16Vector2D_t;

class FVector2D
{
public:
	float x, y;
	inline FVector2D()  : x(0.0f) , y(0.0f) { ; }
	inline FVector2D(float startx, float starty) : x(startx) , y(starty) { ; }
	inline float length() { return sqrt( x*x+y*y ); }
	inline float lengthsqr() { return (x*x+y*y); }
	FVector2D rotate(float angledegree) const;
	FVector2D rotate(Angle angle) const;
	inline F16Vector2D_t to16bit() const;
	
};
inline FVector2D operator+ (const FVector2D& left, const FVector2D& right)
{ return FVector2D(left.x + right.x, left.y + right.y); }
inline FVector2D operator- (const FVector2D& left, const FVector2D& right)
{ return FVector2D(left.x - right.x, left.y - right.y); }
inline FVector2D operator* (const FVector2D& left, const FVector2D& right)
{ return FVector2D(left.x * right.x, left.y * right.y); }
inline FVector2D operator* (const FVector2D& left, const int right)
{ return FVector2D(left.x * right, left.y * right); }
inline FVector2D operator* (const FVector2D& left, const float right)
{ return FVector2D(left.x * right, left.y * right); }
inline FVector2D operator/ (const FVector2D& left, const float right)
{ return FVector2D(left.x / right, left.y / right); }
inline FVector2D operator/ (const FVector2D& left, const FVector2D& right)
{ return FVector2D(left.x / right.x, left.y / right.y); }


#ifdef _arch_dreamcast
#include "sh4.h"

	inline F16Vector2D_t FVector2D::to16bit() const
	{
		return XTRCT( ((*((uint32 *) &x)) >> 16 ), ((*((uint32 *) &y)) >> 0 ) );
	}

	static inline F16Vector2D_t F16Vec(float x, float y)
	{
		union { unsigned int i; float f; } u;
		unsigned int xi, yi;
		u.f = x;
		xi = u.i;
		u.f = y;
		yi = u.i;
		return XTRCT( (xi >> 16 ), (yi >> 0 ) );
	}

#else
	static inline F16Vector2D_t F16Vec(float x, float y)
	{
		union { unsigned int i; float f; } u;
		unsigned int xi, yi;
		u.f = x;
		xi = u.i;
		u.f = y;
		yi = u.i;
		
		return ((xi >> 16 ) << 16) | (yi & 0xffff );
	}

	inline F16Vector2D_t FVector2D::to16bit() const
	{
		return F16Vec(x,y);
	}

#endif

class FVector4D
{
public:
	float x, y, z, w;
	inline FVector4D()  : x(0.0f) , y(0.0f), z(0.0f), w(0.0f) { ; }
	inline FVector4D(float startval) : x(startval) , y(startval), z(startval), w(startval) { ; }
	inline FVector4D(float startx, float starty, float startz, float startw) : x(startx) , y(starty), z(startz), w(startw) { ; }
#ifdef _arch_dreamcast
	inline float dot(const FVector4D& right) { return fipr(x,y,z,w,right.x,right.y,right.z,right.w); }
#else
	inline float dot(const FVector4D& right) { return (x*right.x+y*right.y+z*right.z+w*right.w); }
#endif
	inline float lengthsqr() { return dot(*this); }
	inline float length() { return sqrt( lengthsqr() ); }
};
inline FVector4D operator+ (const FVector4D& left, const FVector4D& right)
{ return FVector4D(left.x + right.x, left.y + right.y, left.z + right.z, left.w + right.w); }
inline FVector4D operator- (const FVector4D& left, const FVector4D& right)
{ return FVector4D(left.x - right.x, left.y - right.y, left.z - right.z, left.w - right.w); }
inline FVector4D operator* (const FVector4D& left, const FVector4D& right)
{ return FVector4D(left.x * right.x, left.y * right.y, left.z * right.z, left.w * right.w); }
inline FVector4D operator* (const FVector4D& left, const float right)
{ return FVector4D(left.x * right, left.y * right, left.z * right, left.w * right); }
inline FVector4D operator/ (const FVector4D& left, const float right)
{ return FVector4D(left.x / right, left.y / right, left.z / right, left.w / right); }
inline FVector4D operator/ (const FVector4D& left, const FVector4D& right)
{ return FVector4D(left.x / right.x, left.y / right.y, left.z / right.z, left.w / right.w); }


class FVector3D
{
public:
	float x, y, z;
	inline FVector3D()  : x(0.0f) , y(0.0f), z(0.0f) { ; }
	inline FVector3D(float startval) : x(startval) , y(startval), z(startval) { ; }
	inline FVector3D(float startx, float starty, float startz) : x(startx) , y(starty), z(startz) { ; }
#ifdef _arch_dreamcast
	inline float dot(const FVector3D& right) { return fipr(x,y,z,0,right.x,right.y,right.z,0); }
#else
	inline float dot(const FVector3D& right) { return (x*right.x+y*right.y+z*right.z); }
#endif
	inline float lengthsqr() { return dot(*this); }
	inline float length() { return sqrt( lengthsqr() ); }
	inline operator btVector3() { return btVector3(x,y,z); }
};
inline FVector3D operator+ (const FVector3D& left, const FVector3D& right)
{ return FVector3D(left.x + right.x, left.y + right.y, left.z + right.z); }
inline FVector3D operator- (const FVector3D& left, const FVector3D& right)
{ return FVector3D(left.x - right.x, left.y - right.y, left.z - right.z); }
inline FVector3D operator* (const FVector3D& left, const FVector3D& right)
{ return FVector3D(left.x * right.x, left.y * right.y, left.z * right.z); }
inline FVector3D operator* (const FVector3D& left, const float right)
{ return FVector3D(left.x * right, left.y * right, left.z * right); }
inline FVector3D operator/ (const FVector3D& left, const float right)
{ return FVector3D(left.x / right, left.y / right, left.z / right); }
inline FVector3D operator/ (const FVector3D& left, const FVector3D& right)
{ return FVector3D(left.x / right.x, left.y / right.y, left.z / right.z); }


#endif

