#define USE_AA

#include <kos.h>
#include <assert.h>
#include <dc/fs_dcload.h>
#include <dc/maple/keyboard.h>
#include <dc/matrix3d.h>

#include <stdlib.h>

#include "common.h"
#include "debug.h"
#include "render.h"
#include "input.h"
#include "profile.h"
#include "pvr_cxt.h"
#include "model.h"

pvr_context context;
pvr_poly_ic_hdr_t texhdr;

#ifdef _arch_dreamcast
	extern uint8 romdisk[];

	KOS_INIT_FLAGS(INIT_DEFAULT | INIT_THD_PREEMPT );
	KOS_INIT_ROMDISK(romdisk);
#endif

void init_pvr()
{
	pvr_init_params_t pvr_params = {
		{ PVR_BINSIZE_32, PVR_BINSIZE_0, PVR_BINSIZE_0, PVR_BINSIZE_0, PVR_BINSIZE_0 },
		(int)1.5*1024*1024,
		0,
		#ifdef USE_AA
		1,	//2x horizontal supersampling on
		#endif
	};
	pvr_init(&pvr_params);
	
	PVR_FSET(PVR_SMALL_CULL, 0.1);
	pvr_set_bg_color(0,0,0.21);
}

void loadtex(const char * fname)
{
	pvr_ptr_t texture;
	uint32 width=1024, height=1024;
	int texsize = width*height/4+2048;
	
	void *vqdata = LoadCompressed("/rd/deadpool.vq.gz");
	assert(vqdata);
	
	texture = pvr_mem_malloc(texsize);
	assert(texture);
	
	pvr_txr_load(vqdata, texture, texsize);
	free(vqdata);
	
	pc_set_default_polygon(&context);
	pc_set_color_format(&context,PC_INTENSITY);
	if (1) {	//use texture
		pc_set_texture(&context, texture, PC_SIZE_1024, PC_SIZE_1024, PC_RGB565, 1, 0, 1);
		pc_set_filter(pc_no_mod(&context), PC_BILINEAR);
	}
}

void manipulate(float *x, float *y, float *z, float speed)
{
	if (cont[0].b.x)	
		*x += cont[0].ax * speed;
	if (cont[0].b.y)
		*y += cont[0].ax * speed;
	if (cont[0].b.b)	
		*z += cont[0].ax * speed;
	if (cont[0].c.l)
		*x = 0;
	if (cont[0].c.u)
		*y = 0;
	if (cont[0].c.r)
		*z = 0;
}

int main()
{	
	#ifdef _arch_dreamcast
		InitGBR();
		trapa_set_handler(1,&dump_regs);
		trapa_set_handler(2,&dump_float_regs);
		trapa_set_handler(3,&dump_float_all_regs);
	#endif
	watchdog_init();
	DBOS("START\n");
	
	init_pvr();
	loadtex("/rd/deadpool.vq");
	
	watchdog_pause();
	DBOS("Loading model\n");
	simple_model *mdl = LoadSimpleModel("/rd/deadpool.bin.gz");
	watchdog_resume();
	if (mdl == NULL) {
		DBOS("Error loading model\n");
		return 0;
	}
	
	DBOS("Addr %p - %i\n",mdl->vertices,31 & (int)mdl->vertices);
	if (31 & (int)mdl->vertices) {
		DBOS("Model data is not aligned to 32 byte boundary, so speed will be reduced\n");
	}
	
	float  tx=0,  ty=11, tz=-10;		//model translation
	float  rx=0,  ry=0,  rz=3.14;	//model rotation
	float lrx=0, lry=0, lrz=0;	//light rotation
	
	//profile_init(50000);
	//profile_go();
	while(1) {
		watchdog_pet();
		
		GetKeyboardEvent();
		GetMouseEvent();
		GetJoystickEvent();
		
		ProfileColorPoint(0, 0, 0);
		
		if ((INKEY == 27) || (INKEY == KBD_KEY_DEL << 8 && CTRL && ALT) || (cont[0].c.s))
			break;
		if ( INKEY == (KBD_KEY_PRINT << 8) )
			SaveScreenshot();
		
		float scale = 1;
		if (cont[0].b.a)
			scale = 4;
		
		if (cont[0].b.sr)
			manipulate(&lrx,&lry,&lrz,0.03 * scale);
		else if (cont[0].b.sl)
			manipulate(&rx,&ry,&rz,0.03 * scale);
		else
			manipulate(&tx,&ty,&tz,0.06 * scale);
		
		if (pvr_wait_ready()) {
			DBOS("!!!PVR_WAIT TIMEOUT!!!\n");
			pvr_stats_t stats;
			pvr_get_stats(&stats);
			DBOS("VBL: %u, FRAMES: %u\n",stats.vbl_count, stats.frame_count);
		}
		
		profile_t rtime = PfStart();
		ProfileColorPoint(0, 255, 00);
		pvr_scene_begin();		
			pvr_list_begin(PVR_LIST_OP_POLY);
				//transform light into model space
				float lx=0,ly=0,lz=1;
				mat_identity();
				mat_rotate_z(-rz);
				mat_rotate_y(-ry);
				mat_rotate_x(-rx);
				mat_rotate_z(lrz);
				mat_rotate_y(lry);
				mat_rotate_x(lrx);
				mat_trans_nodiv_cw(lx,ly,lz,0.0);
				
				//set up model transformation matrix
				mat_identity();
				#ifdef USE_AA
					mat_perspective(640, 240, 1.5, 10000.0, 0.1);
					mat_scale(2,1,1);
				#else
					mat_perspective(320, 240, 1.5, 10000.0, 0.1);
				#endif
				mat_translate(tx,ty,tz);
				mat_rotate(rx,ry,rz);
				
				//draw it
				pvr_prim(&context,32);
				DrawSimpleModel(mdl,lx,ly,lz,0.3);
			pvr_list_finish();
		pvr_scene_finish();
		rtime = PfEnd(rtime);
		
		ProfileColorPoint(255, 0, 0);
		
		if (cont[0].b.sl && cont[0].c.sr) {
			pvr_stats_t stats;
			pvr_get_stats(&stats);
			DBO("SUB: %8.4f, REG: %2i, RND: %2i VTX: %08i, TEX: %i\n",
				PfSecs(rtime),
				stats.reg_last_time, 
				stats.rnd_last_time,
				stats.vtx_buffer_used,
				pvr_mem_available());
			pvr_mem_stats();
		}
	}
	//profile_stop();
	//profile_save("/pc/profile.txt", 0);
	return 0;
}
