.globl _render_strip_mix
! int render_strip_mix(rendertarget, simplelitvert *, int vertcount, float lightx, float lighty,
!	float lightz, float lightw, float ambient)
!r0 sq target/debug ret
!r1 
!r2 sq write pos
!r3 vert strip command/end vert strip command
!r4 sq target [arg] sq toggle xor bit mask
!r5 vertlist read pos [arg]
!r6 vertcount [arg]
!fr0,fr1,fr2,fr3 vertex
!fr4,fr5,fr6,fr7 light vector [arg]
!fr8 ambient [arg]
!fr8,fr9,fr10,fr11 vertex normal
!fr12 ambient
!fr3 1/w
#define x fr0
#define y fr1
#define z fr2
#define w fr3
#define lx fr4
#define ly fr5
#define lz fr6
#define lw fr7
#define nx fr8
#define ny fr9
#define nz fr10
#define nw fr11
#define ambient fr12
#define xy dr0
#define zu dr2
#define lxy dr4
#define lzw dr6
#define nxy dr8
#define nzv dr10
#define uv dr14
#define vec fv0
#define lvec fv4
#define nvec fv8

.align 5
_render_strip_mix:
	pref	@r5	!prefetch vertex
	mov	#224,r3	!load const for vert command
	
	fmov.s	fr12,@-r15	!push to stack so fr12 can hold ambient
	shll16	r3		!shift const to correct value
		
	fmov.s	fr14,@-r15	!push to stack so fr14 can hold u

	mov	r4,r0		!copy target sq to r0
	fmov.s	fr15,@-r15	!push to stack so fr15 can hold v
	
	shll8	r3		!finish const shifting
 	fmov	fr8,ambient	!move ambient to fr12
	
	add	#-32,r0		!switch to other sq
	bra	2f	!skip over submitting empty sq
	fschg		!!!!!switch to double loads
.align 2
1:
	pref	@r0	!submit current sq
	fschg		!!!!!switch to double loads
.align 2
2:
	fmov	@r5+,xy	!load vertex x,y
	add	#32,r0	!switch to other sq

	fmov	@r5+,zu	!load vertex z,u
	mov	r0,r2	!copy target sq to r2 for write pos

	fmov	@r5+,nxy	!load normal x,y
	add	#4*4,r2		!move to vertex write pos to sq's uv
	
	flds	w,fpul
	
	fldi1	w
	
	fmov	@r5+,nzv	!load normal z,v
	
	pref	@r5		!prefetch next vertex
	
	ftrv	xmtrx,vec	!transform
	fsts	fpul,fr14
	
	flds	nw,fpul
	fipr	lvec,nvec	!do dot product between normal and light	
	
	fsts	fpul,fr15
	
	fmov	uv,@r2	!write texture u,v
	add	#4*2,r2	!move write pos to sq intensity
	
	fldi1	z	![PERSPECTIVE] set fr2 to 1 for 1/w calc
	fdiv	w,z	!start 1/w calc
	
	mov.l	r3,@r0	!write vertex command const to sq
	fcmp/gt	ambient, nw	!is the light more than ambient?

	fschg		!!!!!switch to singles
	bt 3f		!if it's less than ambient, make it equal to ambient

	fmov	ambient,nw	!if it's less than zero, make it zero (clamp it so it's more than zero)

3:	dt	r6		!one less vertex to send, check loop for later jump
	fmov	nw,@r2	!write intensity to sq
	
	fmul	z,fr1	!calc y/w
	add	#-4*3,r2	!move sq write pos for vertex z write
	
	fmul	z,fr0	!calc x/w

	fmov	z,@r2	!write 1/w to sq

	fmov	fr1,@-r2	!write y/w to sq
	bf/s	1b		!if there's stuff left, keep sending
	fmov	fr0,@-r2	!write x/w to sq

	!we are on the last triangle, switch to end strip command and submit
	mov #240,r3		!load const for end vert list
	fmov.s @r15+,fr15	!pop stack back

	shll16 r3		!shift const to correct value
	fmov.s @r15+,fr14	!pop stack back

	shll8 r3		!finish const shifting
	fmov.s @r15+,fr12	!pop stack back

	mov.l r3,@r0		!set end of list value at sq start
	
	rts		!return
	pref @r0	![[[DELAYSLOT]]]submit sq, send last triangle to end list

! int render_strip_array_mix(rendertarget, simplelitvert *, int stripcount, size_t *striplens float lightx, float lighty,
!	float lightz, float lightw, float ambient)
#define sq		r0
#define vertcnt		r1
#define dst		r2
#define cmd_vert	r3
#define cmd_eos		r4
#define verts		r5
#define stripcnt	r6
#define striplens	r7
.globl _render_strip_array_mix
.align 5
_render_strip_array_mix:

	mov	#224,cmd_vert	!load const for vert command	
	fmov	fr12,@-r15	!push to stack so fr12 can hold ambient
	
	shll16	cmd_vert	!shift const to correct value	
	fmov	fr14,@-r15	!push to stack so fr14 can hold u
	
	shll8	cmd_vert	!finish const shifting
	fmov	fr15,@-r15	!push to stack so fr15 can hold v
	
	mov	r4,sq		!copy target sq to r0
 	fmov	fr8,ambient	!move ambient to fr12
	
	mov.l	@striplens+, vertcnt
	add	#-32, sq	!switch to other sq

	mov.l	.Leos, cmd_eos
	bra	2f		!skip over submitting empty sq
	fschg			!!!!!switch to double loads
.align 2
1:
	pref	@sq		!submit current sq
	fschg			!!!!!switch to doubles
2:
	pref	@verts	!prefetch next vertex
	
	fmov	@verts+,xy	!load vertex x,y
	add	#32, sq		!switch to other sq

	fmov	@verts+,zu	!load vertex z,u
	mov	sq,dst		!copy target sq to r2 for write pos

	fmov	@verts+,nxy	!load normal x,y
	add	#4*4,dst	!move to vertex write pos to sq's uv
	
	flds	w,fpul		!get u
	
	fldi1	w
	
	fmov	@verts+,nzv	!load normal z,v
	pref	@verts		!prefetch next vertex
	
	ftrv	xmtrx,vec	!transform
	fsts	fpul,fr14
	
	flds	nw,fpul
	fipr	lvec,nvec	!do dot product between normal and light	
	
	fsts	fpul,fr15
	
	fmov	uv,@dst		!write texture u,v
	add	#4*2,dst	!move write pos to sq intensity
	
	fldi1	z
	fdiv	w,z		!start 1/w calc
	
	mov.l	cmd_vert,@sq	!write vertex command const to sq
	fcmp/gt	ambient, nw	!is the light more than ambient?
	
	!
	
	fschg			!!!!!switch to singles
	bt 3f			!if it's less than ambient, make it equal to ambient

	fmov	ambient,nw

3:	fmov	nw,@dst		!write intensity to sq
	fmul	z,y		!calc y/w

	add	#-4*3,dst	!move sq write pos for vertex z write
	fmul	z,x		!calc x/w

	fmov	z,@dst		!write 1/w to sq
	dt	vertcnt		!one less vertex to send

	fmov	y,@-dst		!write y/w to sq
	!

	bf/s	1b
	fmov	x,@-dst		!write x/w to sq
	
!we are on the last triangle of this strip, write end strip command
	dt	stripcnt
	mov.l	cmd_eos,@sq	!set end of list value at sq start
	
	mov.l	@striplens+, vertcnt
	bf	1b
	
	fmov	@r15+,fr15	!pop stack back
	fmov	@r15+,fr14	!pop stack back
	fmov	@r15+,fr12	!pop stack back
	rts
	pref	@sq		!submit last vertex
	
.align 2
.Leos:
	.long 0xf0000000
	
